"use strict;"

var request = require('superagent');

var AlphaApi = require("./alpha_api");
var DefaultApi = require("./default_api");

// Class constructor for the ApiClient
//
// The 'url' should indicate
//
// Options properties:
//
// - `timeout`: Defaults to 0. If nonzero, we'll set a client side timeout, in ms.
//
// Properties defined during initialization;
//
// - `basePath`, defaults to '/api/VERSION'.
function ApiClient(url, options) {
    this.url = url;
    this.basePath = '/api';
    this.timeout = 0;

    Object.defineProperties(this, {
        "apiKey": {
            enumerable: true,
            writable: true
        }
    });
}

// Public method
//
// Creates a handle to the default (stable) set of methods.
ApiClient.prototype.createDefaultApi = function() {
    return new DefaultApi(this);
};

// Public method
//
// Creates a handle to the alpha (unstable) set of methods.
ApiClient.prototype.createAlphaApi = function() {
    return new AlphaApi(this);
};

// Undocumented method
ApiClient.prototype.createUrl = function createUrl(path, pathParams) {
    var url = this.url + this.basePath + path;
    if (pathParams) {
        Object.keys(pathParams).forEach(function(key) {
            url = url.replace("{" + key + "}", encodeURIComponent(pathParams[key]));
        });
    }
    return url;
};

// Undocumented method
ApiClient.prototype.executeMethod = function executeNoBody(method, path, pathParams, query, body, callback) {
    var req = request(method, this.createUrl(path, pathParams));

    if (this.apiKey) {
        req.set('Authorization', this.apiKey);
    }

    if (query) {
        var q = {};
        Object.keys(query).forEach(function(key) {
            if (query[key]) {
                q[key] = query[key];
            }
        });
        req.query(q);
    }

    req.accept('application/json');

    if (0 < this.timeout) {
        req.timeout(this.timeout);
    }

    if (body) {
        req.send(body);
    }

    req.end(callback);
};

module.exports = ApiClient;
